/*
 * BeanContextBeanInfo.java
 *
 * BeanDoc generated on Fri Mar 16 at 12:40:18 EDT 02012.
 * 
 * See BeanContext.java for copyright information.
 */

package org.teatrove.teaapps.contexts;

import java.beans.*;
import java.lang.reflect.*;

import java.util.Vector;

/**
 * The BeanInfo class describing BeanContext. 
 *
 * @author Scott Jappinen
 */
public class BeanContextBeanInfo extends SimpleBeanInfo {

    /** BeanDescriptor returned by the <code>getBeanDescriptor</code> 
        method */
    private BeanDescriptor mBeanDescriptor;

    /** BeanInfo array returned by the <code>getAdditionalBeanInfo</code> 
        method */
    private BeanInfo[] mAdditionalBeanInfo;

    /** MethodDescriptor array returned by the 
        <code>getMethodDescriptors</code> method */
    private MethodDescriptor[] mMethodDescriptors;

    /** PropertyDescriptor array returned by the 
        <code>getPropertyDescriptors</code> method */
    private PropertyDescriptor[] mPropertyDescriptors;
    
    /** The index of the default property */
    private int mDefaultPropertyIndex;


    /**
     * Creates a new BeanContextBeanInfo.
     */
    public BeanContextBeanInfo() {        
                          
        mDefaultPropertyIndex = -1;
        
    }
     
    /**
     * Returns a BeanDescriptor that describes BeanContext.
     */    
    public BeanDescriptor getBeanDescriptor() {
        
        if (mBeanDescriptor == null) {
            mBeanDescriptor = createBeanDescriptor();       
        }

        return mBeanDescriptor;
    }

    /**
     * Returns a BeanInfo array containing BeanInfo object's for:<br>
     * <code>java.lang.Object</code><br>      
     */  
    public BeanInfo[] getAdditionalBeanInfo() {

        if (mAdditionalBeanInfo == null) {
            mAdditionalBeanInfo = createAdditionalBeanInfo();       
        }

        return mAdditionalBeanInfo;
    }

    /**    
     * Returns MethodDescriptors that describe the methods of 
     * BeanContext.  Call the 
     * <code>getAdditionalBeanInfo</code> method to get the 
     * method descriptors of the superclass and interfaces.
     */ 
    public MethodDescriptor[] getMethodDescriptors() {

        if (mMethodDescriptors == null) {
            mMethodDescriptors = createMethodDescriptors();       
        }

        return mMethodDescriptors;
    }

    /**    
     * Returns a PropertyDescriptor array that describes the properties of 
     * BeanContext.
     */ 
    public PropertyDescriptor[] getPropertyDescriptors() {

        if (mPropertyDescriptors == null) {
            mPropertyDescriptors = createPropertyDescriptors();     
        }

        return mPropertyDescriptors;
    }

    /**
     * Return the index of the default property.
     */
    public int getDefaultPropertyIndex() {
        return mDefaultPropertyIndex;
    }


    //
    // Non-public interface
    //

    // Create the bean descriptor for the bean.
    private BeanDescriptor createBeanDescriptor() {                        
        
        BeanDescriptor bd = 
            new BeanDescriptor(BeanContext.class);
        
        bd.setName("BeanContext");
        bd.setDisplayName("BeanContext");
        bd.setShortDescription("The BeanContext is a Tea context that allows dynamic lookup and setting of\n Java beans.  Tea does not support direct dynamic lookup of beans in order to\n be highly performant.  The BeanContext provides this behavior by utilizing a\n specially built dynamic class that performs fast and efficient property\n lookups to decrease the performance impact.\n \n The context works by returning a special object that wraps a given bean and\n uses a special Tea notation to allow map-based lookups.  The lookups may be\n either simple bean properties (ie: name, location, etc) or may be composite\n properties in dot-notation (ie: venue.city.name).  By default, if any portion\n of the composite lookup is <code>null</code>, then <code>null</code> is\n returned.  You can alter this behavior by passing in <code>true</code> for\n the <code>failOnNulls</code> parameter.  In this case, if any portion of the\n composite graph is <code>null</code>, then a {@link BeanContextException}\n is thrown.\n \n Example:\n <code>\n     bean = getBeanMap(myObject)\n     \'Name: \' bean[\'name\']\n </code>");        
        bd.setValue("BeanDoc", "4.1.2");
        
        return bd;
    }

    // Create the additional bean info objects for the bean.
    private BeanInfo[] createAdditionalBeanInfo() {
        
        Vector bis = new Vector();
        BeanInfo bi = null;                
        
        bi = null;
        try {
            // Get the BeanInfo for the superclass
            bi = Introspector.getBeanInfo(java.lang.Object.class);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }        
        
        if (bi != null) {    
            bis.addElement(bi);
        }                            
                        
        BeanInfo[] additionalBeanInfo = null;
        if (bis.size() > 0) {
            additionalBeanInfo = new BeanInfo[bis.size()];
            bis.copyInto(additionalBeanInfo);
        }
        
        return additionalBeanInfo;
    }

    // Create the method descriptor objects for the bean.
    private MethodDescriptor[] createMethodDescriptors() {
                
        Class[] paramTypes = null;
        int paramIndex = 0;  
        ParameterDescriptor[] paramDescriptors = null;
        ParameterDescriptor pd = null;
        
        Vector mds = new Vector();
        Method m = null;
        MethodDescriptor md = null;

           
        
        //
        // getBeanMap 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.Object.class; 
        pd = new ParameterDescriptor();
        pd.setName("bean");
        pd.setDisplayName("java.lang.Object"); 
        pd.setShortDescription("The bean to wrap and return"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = BeanContext.class.getMethod("getBeanMap", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getBeanMap");
            md.setDisplayName("getBeanMap");
            md.setShortDescription("Get an object wrapping the given bean that allows dynamic property\n lookups.\n \n <code>bean = getBeanMap(bean); bean[\'name\']</code>");   
            
            mds.addElement(md);
        }            
           
        
        //
        // getBeanMap 
        //
        paramTypes = new Class[2];
        paramDescriptors = new ParameterDescriptor[2];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.Object.class; 
        pd = new ParameterDescriptor();
        pd.setName("bean");
        pd.setDisplayName("java.lang.Object"); 
        pd.setShortDescription("The bean to wrap and return"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                        
        paramTypes[paramIndex] = boolean.class; 
        pd = new ParameterDescriptor();
        pd.setName("failOnNulls");
        pd.setDisplayName("boolean"); 
        pd.setShortDescription("The state of whether to throw exceptions on null graph"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = BeanContext.class.getMethod("getBeanMap", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getBeanMap");
            md.setDisplayName("getBeanMap");
            md.setShortDescription("Get an object wrapping the given bean that allows dynamic property\n lookups.  The <code>failOnNulls</code> parameter may be used to cause\n an exception to be thrown when portions of the composite graph are\n <code>null</code>.\n \n <code>bean = getBeanMap(bean); bean[\'name\']</code>");   
            
            mds.addElement(md);
        }            
           
        
        //
        // set 
        //
        paramTypes = new Class[3];
        paramDescriptors = new ParameterDescriptor[3];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.Object.class; 
        pd = new ParameterDescriptor();
        pd.setName("object");
        pd.setDisplayName("java.lang.Object"); 
        pd.setShortDescription("The object to set an attribute value for."); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("attributeName");
        pd.setDisplayName("java.lang.String"); 
        pd.setShortDescription("The name of the attribute to set."); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                        
        paramTypes[paramIndex] = java.lang.Object.class; 
        pd = new ParameterDescriptor();
        pd.setName("attributeValue");
        pd.setDisplayName("java.lang.Object"); 
        pd.setShortDescription("The value of the attribute to set."); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = BeanContext.class.getMethod("set", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("set");
            md.setDisplayName("set");
            md.setShortDescription("Method to set attributes of an object.  This method may be \n called to set the values of an object by passing in the object, \n the attribute name to set, and the attribute value.  To get a \n listing of attribute values that can be set for a given object, \n see the createObject method for the object.");   
            
            mds.addElement(md);
        }            
               
                         
        MethodDescriptor[] methodDescriptors = 
                                new MethodDescriptor[mds.size()];
        mds.copyInto(methodDescriptors);
                
        return methodDescriptors;
    }

    // Create the property descriptor objects for the bean.
    private PropertyDescriptor[] createPropertyDescriptors() {
                           
        PropertyDescriptor[] propertyDescriptors = new PropertyDescriptor[0];
        PropertyDescriptor pd = null;
        int propertyIndex = 0;          
        
        if (propertyDescriptors.length == 0) {
            propertyDescriptors = null;    
        }
        
        return propertyDescriptors;
    }

}